@@ -27,7 +27,11 @@ const path = require('path');
 function initializeActionHandler(message) {
     if (message.binary) {
         // The code is a base64-encoded zip file.
-        return unzipInTmpDir(message.code)
+        ext = detectFileType(message.code)
+        if (ext == 'unsupported'){
+            return Promise.reject("There was an error uncompressing the action archive.");
+        }
+        return extractInTmpDir(message.code)
             .then(moduleDir => {
                 let parts = splitMainHandler(message.main);
                 if (parts === undefined) {
@@ -46,6 +50,17 @@ function initializeActionHandler(message) {
                     return Promise.reject('Zipped actions must contain either package.json or index.js at the root.');
                 }
 
+                // install npm modules during init if source code zip doesn´t containt them
+                // check if package.json exists and node_modules don`t
+                if (fs.existsSync('package.json') && !fs.existsSync('./node_modules/')) {
+                    var package_json = JSON.parse(fs.readFileSync('package.json', 'utf8'));
+                    if (package_json.hasOwnProperty('dependencies')) {
+                        if (Object.keys(package_json.dependencies).length > 0) {
+                            exec("npm install")
+                        }
+                    }
+                }
+
                 //  The module to require.
                 let whatToRequire = index !== undefined ? path.join(moduleDir, index) : moduleDir;
                 let handler = eval('require("' + whatToRequire + '").' + main);
@@ -127,21 +142,33 @@ class NodeActionRunner {
  * Note that this makes heavy use of shell commands because the environment is expected
  * to provide the required executables.
  */
-function unzipInTmpDir(zipFileContents) {
+function extractInTmpDir(archiveFileContents) {
     const mkTempCmd = "mktemp -d XXXXXXXX";
     return exec(mkTempCmd).then(tmpDir => {
         return new Promise((resolve, reject) => {
-            const zipFile = path.join(tmpDir, "action.zip");
-            fs.writeFile(zipFile, zipFileContents, "base64", err => {
-                if (!err) resolve(zipFile);
+            ext = detectFileType(archiveFileContents)
+            if (ext == 'unsupported'){
+                reject("There was an error Detecting the File type");
+            }
+            const archiveFile = path.join(tmpDir, "action."+ ext);
+            fs.writeFile(archiveFile, archiveFileContents, "base64", err => {
+                if (!err) resolve(archiveFile);
                 else reject("There was an error reading the action archive.");
             });
         });
-    }).then(zipFile => {
+    }).then(archiveFile => {
         return exec(mkTempCmd).then(tmpDir => {
-            return exec("unzip -qq " + zipFile + " -d " + tmpDir)
+            if (ext === 'zip') {
+                return exec("unzip -qq " + archiveFile + " -d " + tmpDir)
                 .then(res => path.resolve(tmpDir))
                 .catch(error => Promise.reject("There was an error uncompressing the action archive."));
+            } else if (ext === 'tar.gz') {
+                return exec("tar -xzf " + archiveFile + " -C " + tmpDir + " > /dev/null")
+                .then(res => path.resolve(tmpDir))
+                .catch(error => Promise.reject("There was an error uncompressing the action archive."));
+            } else {
+                  return Promise.reject("There was an error uncompressing the action archive.");
+            }
         });
     });
 }
@@ -187,3 +214,22 @@ module.exports = {
     NodeActionRunner,
     initializeActionHandler
 };
+
+// helper function to detect if base64string is zip or tar.gz
+// and returns the file ending
+function detectFileType(base64String) {
+    // Decode the base64 string into binary data
+    const binaryData = Buffer.from(base64String, 'base64');
+
+    // Examine the first few bytes of the binary data to determine the file type
+    const magicNumber = binaryData.slice(0, 4).toString('hex');
+
+    if (magicNumber === '504b0304') {
+      return 'zip';
+    // GZIP: 1f8b0808 maximum compression level,  1f8b0800 default compression
+    } else if (magicNumber === '1f8b0808' || magicNumber === '1f8b0800') {
+      return 'tar.gz';
+    } else {
+        return 'unsupported';
+    }
+}
